/* c/mkcl/config.h.  Generated from configpre.h by configure.  */
/*
    config.h.in -- Template configuration file.
*/
/*
    Copyright (c) 1990, Giuseppe Attardi.
    Copyright (c) 2001, Juan Jose Garcia Ripoll.
    Copyright (c) 2010-2016, Jean-Claude Beaudoin.

    MKCL is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    See file './Copyright' for full details.
*/

#ifndef MKCL_CONFIG_H
#define MKCL_CONFIG_H

#define MKCL_DEBUG_INTERRUPT_MASK 1

/*
 * If MKCL_API has been predefined, that means we are building the core
 * library and, under windows, we must tell the compiler to export
 * extern functions from the shared library.
 * If MKCL_API is not defined, we are simply building an application that
 * uses MKCL and, under windows, we must tell the compiler that certain elements
 * will be imported from a DLL.
 */
#if defined(__MINGW32__) || defined(_MSC_VER) || defined(cygwin)
# define MKCL_WINDOWS 1 /* This one stands as a shorthand for the test above. */
# define MKCL_DLLEXPORT __declspec(dllexport)
# ifdef MKCL_API
#  undef \
   MKCL_API /* Avoid autoconf removing this */
#  define MKCL_API __declspec(dllexport)
# else
#  define MKCL_API __declspec(dllimport)
# endif
#else /* unix and linux */
# define MKCL_DLLEXPORT
# ifdef MKCL_API
#  undef \
   MKCL_API /* Avoid autoconf removing this */
# endif
# define MKCL_API
#endif

/* Decimal number made with the formula: MAJOR * 1000000 + MINOR * 10000 + PATCH */
#define MKCL_VERSION_NUMBER 1010011
#define MKCL_MAJOR_VERSION 1
#define MKCL_MINOR_VERSION 1
#define MKCL_PATCH_LEVEL 11
#define MKCL_VERSION_STRING "1.1.11"
#define MKCL_MAJOR_MINOR_VERSION_STRING "1.1"
#define MKCL_MAJOR_VERSION_STRING "1"
#define MKCL_MINOR_VERSION_STRING "1"
#define MKCL_PATCH_LEVEL_STRING "11"

/*
 * FEATURES LINKED IN
 */

/* Use GNU Multiple Precision library for bignums                       */
#include <gmp.h>

#if defined(_MSC_VER) || defined(__MINGW32__)
# define MKCL_WSOCK
#endif

/* Foreign functions interface						*/
/* #undef MKCL_FFI */

/* Allow STREAM operations to work on arbitrary objects			*/
/* #undef MKCL_CLOS_STREAMS */

/* Stack grows downwards						*/
#define MKCL_DOWN_STACK 1

/* We have non-portable implementation of FFI calls			*/
/* #undef MKCL_DYNAMIC_FFI */

/* We use hierarchical package names, like in Allegro CL                */
#define MKCL_RELATIVE_PACKAGE_NAMES 1


/* Integer types                        				*/
#include <stdint.h>
#define mkcl_uint8_t uint8_t
#define mkcl_int8_t int8_t
#define mkcl_uint16_t uint16_t
#define mkcl_int16_t int16_t
#define mkcl_uint32_t uint32_t
#define mkcl_int32_t int32_t
#define mkcl_uint64_t uint64_t
#define mkcl_int64_t int64_t
#define mkcl_long_long_t long long
#define mkcl_ulong_long_t unsigned long long

/*
 * C TYPES AND SYSTEM LIMITS
 */
#include <limits.h>

/*
 * The integer type
 *
 * mkcl_word must be an integer type, large enough to hold a pointer.
 * Ideally, according to the ISOC99 standard, we should use intptr_t,
 * but the required headers are not present in all systems. Hence we
 * use autoconf to guess the following values.
 */
#define MKCL_INT_BITS           32
#define MKCL_LONG_BITS          64
#define MKCL_WORD_BITS          64
#define MKCL_MOST_POSITIVE_FIXNUM	((mkcl_word)2305843009213693951)
#define MKCL_MOST_NEGATIVE_FIXNUM	((mkcl_word)-2305843009213693952)
#define MKCL_MOST_POSITIVE_FIXNUM_RAW	(2305843009213693951)
#define MKCL_MOST_NEGATIVE_FIXNUM_RAW	(-2305843009213693952)

typedef long int mkcl_word;
typedef unsigned long int mkcl_index;
typedef unsigned long int mkcl_hashkey;

/*
 * The character type
 */
#define	MKCL_CHAR_CODE_LIMIT	1114112	/*  unicode character code limit (0x110000) */
#define	MKCL_BASE_CHAR_CODE_LIMIT 256	/*  unicode character code limit (0x100) */

/*
 * Array limits
 */
#define	MKCL_ARANKLIM	64		/*  array rank limit  		*/
#define	MKCL_ADIMLIM		2305843009213693951	/*  array dimension limit	*/
#define	MKCL_ATOTLIM		2305843009213693951	/*  array total limit		*/

/*
 * Function limits.
 *
 * In general, any of these limits must fit in a "mkcl_word".
 */
/*	Maximum number of function arguments (arbitrary)		*/
#define MKCL_CALL_ARGUMENTS_LIMIT MKCL_MOST_POSITIVE_FIXNUM_RAW  /* Used to be 65536. JCB */

/*	Maximum number of required arguments				*/
#define MKCL_LAMBDA_PARAMETERS_LIMIT MKCL_CALL_ARGUMENTS_LIMIT

/*	Numb. of args. which will be passed using the C stack		*/
/*	See cmplam.lsp if you change this value				*/
#define MKCL_C_ARGUMENTS_LIMIT 64

/*	Maximum number of output arguments (>= MKCL_C_ARGUMENTS_LIMIT)	*/
#define MKCL_MULTIPLE_VALUES_LIMIT 64

/* A setjmp that does not save signals					*/
#if __unix__
/* Setting the second argument of sigsetjmp to TRUE causes a severe 
   speed performance degradation of the catch/throw facility.
   Can it be justified? JCB */
# define mkcl_setjmp(buf) sigsetjmp(buf, FALSE)
# define mkcl_longjmp(buf, val)	siglongjmp(buf, val)
# define mkcl_jmp_buf sigjmp_buf
#else
# define mkcl_setjmp	_setjmp
# define mkcl_longjmp	_longjmp
# define mkcl_jmp_buf jmp_buf
#endif

/*
 * Structure/Instance limits. The index to a slot must fit in the
 * "int" type. We also require MKCL_SLOTS_LIMIT <= CALL_ARGUMENTS_LIMIT
 * because constructors typically require as many arguments as slots,
 * or more.
 */
#define MKCL_SLOTS_LIMIT	32768

/* compiler understands long double                                     */
/* #undef HAVE_LONG_DOUBLE */
#if 0 /* defined(unix) */ /* def HAVE_LONG_DOUBLE */
/* This is commented out because printing of long double based long-float
   is still unstable. 2011/02/07 JCB
*/
# define MKCL_LONG_FLOAT
#endif
/* compiler understands complex                                         */
/* #undef HAVE_DOUBLE_COMPLEX */
/* #undef HAVE_FLOAT_COMPLEX */


#endif /* MKCL_CONFIG_H */

